<?php

namespace Drupal\entity_type_behaviors\Helper;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\entity_type_behaviors\Config\BehaviorConfigFactory;
use Drupal\entity_type_behaviors\EntityTypeBehaviorManager;

/**
 * Class EntityTypeBehaviorHelper.
 *
 * @package Drupal\entity_type_behaviors\Helper
 */
class EntityTypeBehaviorHelper {


  /**
   * The entity type behavior manager.
   *
   * @var \Drupal\entity_type_behaviors\EntityTypeBehaviorManager
   */
  protected $entityTypeBehaviorManager;

  /**
   * Helps us with storing and retrieving config.
   *
   * @var \Drupal\entity_type_behaviors\Config\BehaviorConfigFactory
   */
  protected $configFactory;

  /**
   * EntityTypeFormHelper constructor.
   *
   * @param \Drupal\entity_type_behaviors\EntityTypeBehaviorManager $entityTypeBehaviorManager
   *   The entity type behavior manager.
   * @param \Drupal\entity_type_behaviors\Config\BehaviorConfigFactory $configFactory
   *   Helps us with storing and retrieving the behavior config.
   */
  public function __construct(
    EntityTypeBehaviorManager $entityTypeBehaviorManager,
    BehaviorConfigFactory $configFactory
  ) {
    $this->entityTypeBehaviorManager = $entityTypeBehaviorManager;
    $this->configFactory = $configFactory;
  }

  /**
   * Gets the behaviors for an entity.
   *
   * Returns NULL if the entity is not compatible.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to return behaviors.
   *
   * @return array|null
   *   An array containing behaviors.
   */
  public function getBehaviorsForEntity(EntityInterface $entity) : ?array {
    $entityType = $entity->getEntityTypeId();

    $factory = $this->configFactory;
    $entityTypeBundles = $factory->getConfiguredEntityTypesAndBundles();

    if (empty($entityTypeBundles) || !isset($entityTypeBundles[$entityType])) {
      return NULL;
    }

    if (!$entity instanceof FieldableEntityInterface || !$entity->hasField('behaviors')) {
      return NULL;
    }

    $fieldList = $entity->get('behaviors');

    /** @var \Drupal\entity_type_behaviors\Plugin\Field\FieldType\EntityTypeBehaviorItem $behaviorsItem */
    if ($fieldList->isEmpty() || !$behaviorsItem = $fieldList->first()) {
      return [];
    }

    $behaviors = $behaviorsItem->getUnserializedValue() ?? [];
    return $behaviors;
  }

}
